@@ -9,11 +9,8 @@
 #include "H264.h"
 #include "H265.h"
 #include "MediaData.h"
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/EndianUtils.h"
-#include "mozilla/ResultExtensions.h"
 #include "mozilla/Try.h"
-#include "mozilla/Unused.h"
 
 mozilla::LazyLogModule gAnnexB("AnnexB");
 
@@ -47,8 +44,7 @@ Result<Ok, nsresult> AnnexB::ConvertAVCCSampleToAnnexB(
   ByteWriter<BigEndian> writer(tmp);
 
   while (reader.Remaining() >= 4) {
-    uint32_t nalLen;
-    MOZ_TRY_VAR(nalLen, reader.ReadU32());
+    uint32_t nalLen = MOZ_TRY(reader.ReadU32());
     const uint8_t* p = reader.Read(nalLen);
 
     if (!writer.Write(kAnnexBDelimiter, std::size(kAnnexBDelimiter))) {
@@ -122,8 +118,7 @@ Result<Ok, nsresult> AnnexB::ConvertHVCCSampleToAnnexB(
   nsTArray<uint8_t> tmp;
   ByteWriter<BigEndian> writer(tmp);
   while (reader.Remaining() >= 4) {
-    uint32_t nalLen;
-    MOZ_TRY_VAR(nalLen, reader.ReadU32());
+    uint32_t nalLen = MOZ_TRY(reader.ReadU32());
     const uint8_t* p = reader.Read(nalLen);
     if (!writer.Write(kAnnexBDelimiter, std::size(kAnnexBDelimiter))) {
       LOG("Failed to write kAnnexBDelimiter, OOM?");
@@ -204,9 +199,9 @@ already_AddRefed<mozilla::MediaByteBuffer> AnnexB::ConvertAVCCExtraDataToAnnexB(
   const uint8_t* ptr = reader.Read(5);
   if (ptr && ptr[0] == 1) {
     // Append SPS then PPS
-    Unused << reader.ReadU8().map(
+    (void)reader.ReadU8().map(
         [&](uint8_t x) { return ConvertSPSOrPPS(reader, x & 31, annexB); });
-    Unused << reader.ReadU8().map(
+    (void)reader.ReadU8().map(
         [&](uint8_t x) { return ConvertSPSOrPPS(reader, x, annexB); });
     // MP4Box adds extra bytes that we ignore. I don't know what they do.
   }
@@ -234,8 +229,7 @@ already_AddRefed<mozilla::MediaByteBuffer> AnnexB::ConvertHVCCExtraDataToAnnexB(
 Result<mozilla::Ok, nsresult> AnnexB::ConvertSPSOrPPS(
     BufferReader& aReader, uint8_t aCount, mozilla::MediaByteBuffer* aAnnexB) {
   for (int i = 0; i < aCount; i++) {
-    uint16_t length;
-    MOZ_TRY_VAR(length, aReader.ReadU16());
+    uint16_t length = MOZ_TRY(aReader.ReadU16());
 
     const uint8_t* ptr = aReader.Read(length);
     if (!ptr) {
@@ -255,46 +249,44 @@ static Result<Ok, nsresult> FindStartCodeInternal(BufferReader& aBr) {
     if (res.isOk() && (res.unwrap() == 0x000001)) {
       return Ok();
     }
-    mozilla::Unused << aBr.Read(1);
+    (void)aBr.Read(1);
   }
 
   while (aBr.Remaining() >= 6) {
-    uint32_t x32;
-    MOZ_TRY_VAR(x32, aBr.PeekU32());
+    uint32_t x32 = MOZ_TRY(aBr.PeekU32());
     if ((x32 - 0x01010101) & (~x32) & 0x80808080) {  // Has 0x00 byte(s).
       if ((x32 >> 8) == 0x000001) {                  // 0x000001??
         return Ok();
       }
       if ((x32 & 0xffffff) == 0x000001) {  // 0x??000001
-        mozilla::Unused << aBr.Read(1);
+        (void)aBr.Read(1);
         return Ok();
       }
       if ((x32 & 0xff) == 0) {  // 0x??????00
         const uint8_t* p = aBr.Peek(1);
         if ((x32 & 0xff00) == 0 && p[4] == 1) {  // 0x????0000,01
-          mozilla::Unused << aBr.Read(2);
+          (void)aBr.Read(2);
           return Ok();
         }
         if (p[4] == 0 && p[5] == 1) {  // 0x??????00,00,01
-          mozilla::Unused << aBr.Read(3);
+          (void)aBr.Read(3);
           return Ok();
         }
       }
     }
-    mozilla::Unused << aBr.Read(4);
+    (void)aBr.Read(4);
   }
 
   while (aBr.Remaining() >= 3) {
-    uint32_t data;
-    MOZ_TRY_VAR(data, aBr.PeekU24());
+    uint32_t data = MOZ_TRY(aBr.PeekU24());
     if (data == 0x000001) {
       return Ok();
     }
-    mozilla::Unused << aBr.Read(1);
+    (void)aBr.Read(1);
   }
 
   // No start code were found; Go back to the beginning.
-  mozilla::Unused << aBr.Seek(offset);
+  (void)aBr.Seek(offset);
   return Err(NS_ERROR_FAILURE);
 }
 
@@ -309,13 +301,12 @@ static Result<Ok, nsresult> FindStartCode(BufferReader& aBr,
   if (aBr.Offset()) {
     // Check if it's 4-bytes start code
     aBr.Rewind(1);
-    uint8_t data;
-    MOZ_TRY_VAR(data, aBr.ReadU8());
+    uint8_t data = MOZ_TRY(aBr.ReadU8());
     if (data == 0) {
       aStartSize = 4;
     }
   }
-  mozilla::Unused << aBr.Read(3);
+  (void)aBr.Read(3);
   return Ok();
 }
 
@@ -526,16 +517,16 @@ AnnexB::ConvertNALUTo4BytesNALU(mozilla::MediaRawData* aSample,
     uint32_t nalLen;
     switch (aNALUSize) {
       case 1:
-        MOZ_TRY_VAR(nalLen, reader.ReadU8());
+        nalLen = MOZ_TRY(reader.ReadU8());
         break;
       case 2:
-        MOZ_TRY_VAR(nalLen, reader.ReadU16());
+        nalLen = MOZ_TRY(reader.ReadU16());
         break;
       case 3:
-        MOZ_TRY_VAR(nalLen, reader.ReadU24());
+        nalLen = MOZ_TRY(reader.ReadU24());
         break;
       case 4:
-        MOZ_TRY_VAR(nalLen, reader.ReadU32());
+        nalLen = MOZ_TRY(reader.ReadU32());
         break;
       default:
         MOZ_ASSERT_UNREACHABLE("Bytes of the NAL body length must be in [1,4]");
