@@ -17,6 +17,7 @@
 #  include <mach/mach_traps.h>
 #  include "base/rand_util.h"
 #  include "chrome/common/mach_ipc_mac.h"
+#  include "mozilla/StaticPrefs_layers.h"
 #  include "mozilla/StaticPrefs_media.h"
 #endif
 #ifdef MOZ_WIDGET_COCOA
@@ -86,6 +87,7 @@
 #if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
 #  include "GMPProcessParent.h"
 #  include "nsMacUtilsImpl.h"
+#  include "mozilla/gfx/GPUProcessHost.h"
 #endif
 
 #include "mozilla/ipc/UtilityProcessHost.h"
@@ -179,9 +181,8 @@ class BaseProcessLauncher {
     if (ShouldHaveDirectoryService()) {
       // "Current process directory" means the app dir, not the current
       // working dir or similar.
-      mozilla::Unused
-          << nsDirectoryService::gService->GetCurrentProcessDirectory(
-                 getter_AddRefs(mAppDir));
+      (void)nsDirectoryService::gService->GetCurrentProcessDirectory(
+          getter_AddRefs(mAppDir));
     }
   }
 
@@ -199,13 +200,11 @@ class BaseProcessLauncher {
   virtual ~BaseProcessLauncher() = default;
 
   RefPtr<ProcessLaunchPromise> PerformAsyncLaunch();
-  RefPtr<ProcessLaunchPromise> FinishLaunch();
 
   // Overrideable hooks. If superclass behavior is invoked, it's always at the
   // top of the override.
   virtual Result<Ok, LaunchError> DoSetup();
-  virtual RefPtr<ProcessHandlePromise> DoLaunch() = 0;
-  virtual Result<Ok, LaunchError> DoFinishLaunch();
+  virtual RefPtr<ProcessLaunchPromise> DoLaunch() = 0;
 
   void MapChildLogging();
 
@@ -240,7 +239,6 @@ class BaseProcessLauncher {
   bool mDisableOSActivityMode;
 #endif
   LaunchResults mResults = LaunchResults();
-  TimeStamp mStartTimeStamp = TimeStamp::Now();
   char mInitialChannelIdString[NSID_LENGTH];
   char mChildIDString[32];
 
@@ -258,8 +256,7 @@ class WindowsProcessLauncher : public BaseProcessLauncher {
 
  protected:
   virtual Result<Ok, LaunchError> DoSetup() override;
-  virtual RefPtr<ProcessHandlePromise> DoLaunch() override;
-  virtual Result<Ok, LaunchError> DoFinishLaunch() override;
+  virtual RefPtr<ProcessLaunchPromise> DoLaunch() override;
 
   mozilla::Maybe<CommandLine> mCmdLine;
 #  ifdef MOZ_SANDBOX
@@ -281,7 +278,7 @@ class PosixProcessLauncher : public BaseProcessLauncher {
 
  protected:
   virtual Result<Ok, LaunchError> DoSetup() override;
-  virtual RefPtr<ProcessHandlePromise> DoLaunch() override;
+  virtual RefPtr<ProcessLaunchPromise> DoLaunch() override;
 
   nsCOMPtr<nsIFile> mProfileDir;
 };
@@ -302,7 +299,7 @@ class MacProcessLauncher : public PosixProcessLauncher {
   }
 
  protected:
-  virtual Result<Ok, LaunchError> DoFinishLaunch() override;
+  virtual RefPtr<ProcessLaunchPromise> DoLaunch() override;
 
   std::string mMachConnectionName;
   // We add a mach port to the command line so the child can communicate its
@@ -320,7 +317,7 @@ class AndroidProcessLauncher : public PosixProcessLauncher {
       : PosixProcessLauncher(aHost, std::move(aExtraOpts)) {}
 
  protected:
-  virtual RefPtr<ProcessHandlePromise> DoLaunch() override;
+  virtual RefPtr<ProcessLaunchPromise> DoLaunch() override;
   RefPtr<ProcessHandlePromise> LaunchAndroidService(
       const GeckoProcessType aType, const geckoargs::ChildProcessArgs& args);
 };
@@ -348,7 +345,7 @@ class IosProcessLauncher : public PosixProcessLauncher {
       : PosixProcessLauncher(aHost, std::move(aExtraOpts)) {}
 
  protected:
-  virtual RefPtr<ProcessHandlePromise> DoLaunch() override;
+  virtual RefPtr<ProcessLaunchPromise> DoLaunch() override;
 
   DarwinObjectPtr<xpc_object_t> mBootstrapMessage;
 };
@@ -526,16 +523,23 @@ mozilla::BinPathType BaseProcessLauncher::GetPathToBinary(
   }
 
 #ifdef MOZ_WIDGET_COCOA
-  // The GMP child process runs via the Media Plugin Helper executable
-  // which is a clone of plugin-container allowing for GMP-specific
-  // codesigning entitlements.
+  // To support different child process types having different codesigning
+  // entitlements, different executables are used to run some child process
+  // types.
   nsCString bundleName;
   std::string executableLeafName;
   if (processType == GeckoProcessType_GMPlugin &&
       mozilla::StaticPrefs::media_plugin_helper_process_enabled()) {
+    // Use the media plugin helper executable
     bundleName = MOZ_EME_PROCESS_BUNDLENAME;
     executableLeafName = MOZ_EME_PROCESS_NAME_BRANDED;
+  } else if (processType == GeckoProcessType_GPU &&
+             mozilla::StaticPrefs::layers_gpu_process_executable_enabled()) {
+    // Use the GPU helper executable
+    bundleName = MOZ_GPU_PROCESS_BUNDLENAME;
+    executableLeafName = MOZ_GPU_PROCESS_NAME_BRANDED;
   } else {
+    // the default child process executable
     bundleName = MOZ_CHILD_PROCESS_BUNDLENAME;
     executableLeafName = MOZ_CHILD_PROCESS_NAME;
   }
@@ -668,8 +672,8 @@ bool GeckoChildProcessHost::PrepareLaunch(
 #  if defined(MOZ_SANDBOX)
   if (ShouldHaveDirectoryService() &&
       mProcessType != GeckoProcessType_GMPlugin) {
-    mozilla::Unused << NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
-                                              getter_AddRefs(mProfileDir));
+    (void)NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
+                                 getter_AddRefs(mProfileDir));
   }
 #  endif
 #endif
@@ -725,6 +729,7 @@ bool GeckoChildProcessHost::AsyncLaunch(
 
   RefPtr<BaseProcessLauncher> launcher =
       new ProcessLauncher(this, std::move(aExtraOpts));
+  TimeStamp startTimeStamp = TimeStamp::Now();
 #ifdef ALLOW_GECKO_CHILD_PROCESS_ARCH
   launcher->SetLaunchArchitecture(mLaunchArch);
 #endif
@@ -739,7 +744,7 @@ bool GeckoChildProcessHost::AsyncLaunch(
           &BaseProcessLauncher::Launch, this)
           ->Then(
               XRE_GetAsyncIOEventTarget(), __func__,
-              [this](LaunchResults&& aResults) {
+              [this, startTimeStamp](LaunchResults&& aResults) {
                 {
                   {
                     mozilla::AutoWriteLock handleLock(mHandleLock);
@@ -787,6 +792,9 @@ bool GeckoChildProcessHost::AsyncLaunch(
                   this->mSandboxBroker = std::move(aResults.mSandboxBroker);
 #endif
 
+                  glean::process::child_launch.AccumulateRawDuration(
+                      TimeStamp::Now() - startTimeStamp);
+
                   MonitorAutoLock lock(mMonitor);
                   // The OnChannel{Connected,Error} may have already advanced
                   // the state.
@@ -947,17 +955,14 @@ void BaseProcessLauncher::GetChildLogName(const char* origLogName,
   buffer.AppendASCII(mChildIDString);
 }
 
-// Windows needs a single dedicated thread for process launching,
-// because of thread-safety restrictions/assertions in the sandbox
-// code.
+// We use a single dedicated thread for process launching. This is required on
+// Windows (due to sandboxing code), Android (due to the java code) and Linux
+// (due to ForkServerChild). Using a single thread is not required on macOS or
+// iOS, but we use one to keep the implementation consistent.
 //
-// Android also needs a single dedicated thread to simplify thread
-// safety in java.
-//
-// Fork server needs a dedicated thread for accessing
-// |ForkServiceChild|.
-#if defined(XP_WIN) || defined(MOZ_WIDGET_ANDROID) || \
-    defined(MOZ_ENABLE_FORKSERVER)
+// The only tier-1 platform where we do not need a dedicated thread is macOS,
+// however we use a dedicated thread there as well for consistency and
+// simplicity of implementation.
 
 static mozilla::StaticMutex gIPCLaunchThreadMutex;
 static mozilla::StaticRefPtr<nsIThread> gIPCLaunchThread
@@ -985,11 +990,11 @@ IPCLaunchThreadObserver::Observe(nsISupports* aSubject, const char* aTopic,
   }
 
   nsresult rv = thread ? thread->Shutdown() : NS_OK;
-  mozilla::Unused << NS_WARN_IF(NS_FAILED(rv));
+  (void)NS_WARN_IF(NS_FAILED(rv));
   return rv;
 }
 
-nsCOMPtr<nsIEventTarget> GetIPCLauncher() {
+nsCOMPtr<nsISerialEventTarget> GetIPCLauncher() {
   StaticMutexAutoLock lock(gIPCLaunchThreadMutex);
   if (!gIPCLaunchThread) {
     nsCOMPtr<nsIThread> thread;
@@ -1006,25 +1011,11 @@ nsCOMPtr<nsIEventTarget> GetIPCLauncher() {
     }
   }
 
-  nsCOMPtr<nsIEventTarget> thread = gIPCLaunchThread.get();
+  nsCOMPtr<nsISerialEventTarget> thread = gIPCLaunchThread.get();
   MOZ_DIAGNOSTIC_ASSERT(thread);
   return thread;
 }
 
-#else  // defined(XP_WIN) || defined(MOZ_WIDGET_ANDROID) ||
-       // defined(MOZ_ENABLE_FORKSERVER)
-
-// Other platforms use an on-demand thread pool.
-
-nsCOMPtr<nsIEventTarget> GetIPCLauncher() {
-  nsCOMPtr<nsIEventTarget> pool =
-      mozilla::SharedThreadPool::Get("IPC Launch"_ns);
-  MOZ_DIAGNOSTIC_ASSERT(pool);
-  return pool;
-}
-
-#endif  // XP_WIN || MOZ_WIDGET_ANDROID || MOZ_ENABLE_FORKSERVER
-
 void
 #if defined(XP_WIN)
 AddAppDirToCommandLine(CommandLine& aCmdLine, nsIFile* aAppDir)
@@ -1056,7 +1047,7 @@ AddAppDirToCommandLine(geckoargs::ChildProcessArgs& aCmdLine,
       // tests require startup with a missing profile dir.
       // For users, almost universally, the profile will be in
       // the home directory and normalization isn't required.
-      mozilla::Unused << aProfileDir->Normalize();
+      (void)aProfileDir->Normalize();
       nsAutoCString path;
       MOZ_ALWAYS_SUCCEEDS(aProfileDir->GetNativePath(path));
       geckoargs::sProfile.Put(path.get(), aCmdLine);
@@ -1080,15 +1071,20 @@ RefPtr<ProcessLaunchPromise> BaseProcessLauncher::PerformAsyncLaunch() {
   if (aError.isErr()) {
     return ProcessLaunchPromise::CreateAndReject(aError.unwrapErr(), __func__);
   }
-  RefPtr<BaseProcessLauncher> self = this;
   return DoLaunch()->Then(
       mLaunchThread, __func__,
-      [self](base::ProcessHandle aHandle) {
-        self->mResults.mHandle = aHandle;
-        return self->FinishLaunch();
-      },
-      [](LaunchError aError) {
-        return ProcessLaunchPromise::CreateAndReject(aError, __func__);
+      [self =
+           RefPtr{this}](ProcessLaunchPromise::ResolveOrRejectValue&& aResult) {
+        // Explicitly destroy any outstanding references to HANDLEs which may be
+        // held by mChildArgs before resolving.
+        //
+        // NOTE: This is technically redundant, as it will happen naturally as
+        // the `BaseProcessLauncher` is destroyed, but this step makes it
+        // explicit. If we leaked one of these HANDLEs, we could fail to detect
+        // a child process shutting down or crashing.
+        self->mChildArgs = {};
+        return ProcessLaunchPromise::CreateAndResolveOrReject(
+            std::move(aResult), __func__);
       });
 }
 
@@ -1165,16 +1161,6 @@ void BaseProcessLauncher::MapChildLogging() {
   }
 }
 
-Result<Ok, LaunchError> BaseProcessLauncher::DoFinishLaunch() {
-  // We're in the parent and the child was launched. Clean up any FDs which were
-  // transferred to the child in the parent as soon as possible, which will
-  // allow the parent to detect when the child closes its handle (either due to
-  // normal exit or due to crash).
-  mChildArgs.mFiles.clear();
-
-  return Ok();
-}
-
 #if defined(MOZ_WIDGET_GTK)
 Result<Ok, LaunchError> LinuxProcessLauncher::DoSetup() {
   Result<Ok, LaunchError> aError = PosixProcessLauncher::DoSetup();
@@ -1334,35 +1320,44 @@ Result<Ok, LaunchError> PosixProcessLauncher::DoSetup() {
 #endif  // XP_UNIX
 
 #if defined(MOZ_WIDGET_ANDROID)
-RefPtr<ProcessHandlePromise> AndroidProcessLauncher::DoLaunch() {
-  return LaunchAndroidService(mProcessType, mChildArgs);
+RefPtr<ProcessLaunchPromise> AndroidProcessLauncher::DoLaunch() {
+  return LaunchAndroidService(mProcessType, mChildArgs)
+      ->Then(
+          mLaunchThread, __func__,
+          [self = RefPtr{this}](ProcessHandle aHandle) {
+            self->mResults.mHandle = aHandle;
+            return ProcessLaunchPromise::CreateAndResolve(
+                std::move(self->mResults), __func__);
+          },
+          [](LaunchError aError) {
+            return ProcessLaunchPromise::CreateAndReject(aError, __func__);
+          });
 }
 #endif  // MOZ_WIDGET_ANDROID
 
 #ifdef XP_UNIX
-RefPtr<ProcessHandlePromise> PosixProcessLauncher::DoLaunch() {
-  ProcessHandle handle = 0;
+RefPtr<ProcessLaunchPromise> PosixProcessLauncher::DoLaunch() {
   Result<Ok, LaunchError> result = Err(LaunchError{"Launch not attempted"});
 #  ifdef MOZ_ENABLE_FORKSERVER
   if (mProcessType != GeckoProcessType_ForkServer && ForkServiceChild::Get()) {
     result = ForkServiceChild::Get()->SendForkNewSubprocess(
-        std::move(mChildArgs), std::move(*mLaunchOptions), &handle);
+        std::move(mChildArgs), std::move(*mLaunchOptions), &mResults.mHandle);
   } else
 #  endif
   {
-    result =
-        base::LaunchApp(mChildArgs.mArgs, std::move(*mLaunchOptions), &handle);
+    result = base::LaunchApp(mChildArgs.mArgs, std::move(*mLaunchOptions),
+                             &mResults.mHandle);
   }
 
   if (result.isErr()) {
-    return ProcessHandlePromise::CreateAndReject(result.unwrapErr(), __func__);
+    return ProcessLaunchPromise::CreateAndReject(result.unwrapErr(), __func__);
   }
-  return ProcessHandlePromise::CreateAndResolve(handle, __func__);
+  return ProcessLaunchPromise::CreateAndResolve(std::move(mResults), __func__);
 }
 #endif  // XP_UNIX
 
 #ifdef XP_IOS
-RefPtr<ProcessHandlePromise> IosProcessLauncher::DoLaunch() {
+RefPtr<ProcessLaunchPromise> IosProcessLauncher::DoLaunch() {
   ExtensionKitProcess::Kind kind = ExtensionKitProcess::Kind::WebContent;
   if (mProcessType == GeckoProcessType_GPU) {
     kind = ExtensionKitProcess::Kind::Rendering;
@@ -1421,7 +1416,7 @@ RefPtr<ProcessHandlePromise> IosProcessLauncher::DoLaunch() {
   xpc_dictionary_set_value(bootstrapMessage.get(), "sendRights",
                            sendRightsArray.get());
 
-  auto promise = MakeRefPtr<ProcessHandlePromise::Private>(__func__);
+  auto promise = MakeRefPtr<ProcessLaunchPromise::Private>(__func__);
   ExtensionKitProcess::StartProcess(kind, [self = RefPtr{this}, promise,
                                            bootstrapMessage =
                                                std::move(bootstrapMessage)](
@@ -1500,7 +1495,8 @@ RefPtr<ProcessHandlePromise> IosProcessLauncher::DoLaunch() {
             return;
           }
 
-          promise->Resolve(pid, __func__);
+          self->mResults.mHandle = pid;
+          promise->Resolve(std::move(self->mResults), __func__);
         });
   });
 
@@ -1509,22 +1505,33 @@ RefPtr<ProcessHandlePromise> IosProcessLauncher::DoLaunch() {
 #endif
 
 #ifdef XP_MACOSX
-Result<Ok, LaunchError> MacProcessLauncher::DoFinishLaunch() {
-  Result<Ok, LaunchError> aError = PosixProcessLauncher::DoFinishLaunch();
-  if (aError.isErr()) {
-    return aError;
-  }
-
-  MOZ_ASSERT(mParentRecvPort, "should have been configured during DoSetup()");
-
-  // Wait for the child process to send us its 'task_t' data, then send it the
-  // mach send/receive rights which are being passed on the commandline.
-  const int kTimeoutMs = 10000;
-  MOZ_TRY(MachHandleProcessCheckIn(
-      mParentRecvPort.get(), base::GetProcId(mResults.mHandle), kTimeoutMs,
-      mChildArgs.mSendRights, &mResults.mChildTask));
-
-  return Ok();
+RefPtr<ProcessLaunchPromise> MacProcessLauncher::DoLaunch() {
+  return PosixProcessLauncher::DoLaunch()->Then(
+      XRE_GetAsyncIOEventTarget(), __func__,
+      [self = RefPtr{this}](
+          LaunchResults&& aResults) -> RefPtr<ProcessLaunchPromise> {
+        // Wait for the child process to send us its 'task_t' data, then
+        // send it the mach send/receive rights which are being passed on
+        // the commandline.
+        return MachHandleProcessCheckIn(std::move(self->mParentRecvPort),
+                                        base::GetProcId(aResults.mHandle),
+                                        mozilla::TimeDuration::FromSeconds(10),
+                                        std::move(self->mChildArgs.mSendRights))
+            ->Then(
+                XRE_GetAsyncIOEventTarget(), __func__,
+                [self, results = std::move(aResults)](task_t aTask) mutable {
+                  results.mChildTask = aTask;
+                  return ProcessLaunchPromise::CreateAndResolve(
+                      std::move(results), __func__);
+                },
+                [](LaunchError aError) {
+                  return ProcessLaunchPromise::CreateAndReject(aError,
+                                                               __func__);
+                });
+      },
+      [](LaunchError aError) {
+        return ProcessLaunchPromise::CreateAndReject(aError, __func__);
+      });
 }
 #endif  // XP_MACOSX
 
@@ -1690,8 +1697,7 @@ Result<Ok, LaunchError> WindowsProcessLauncher::DoSetup() {
   return Ok();
 }
 
-RefPtr<ProcessHandlePromise> WindowsProcessLauncher::DoLaunch() {
-  ProcessHandle handle = 0;
+RefPtr<ProcessLaunchPromise> WindowsProcessLauncher::DoLaunch() {
 #  ifdef MOZ_SANDBOX
   if (mUseSandbox) {
     const IMAGE_THUNK_DATA* cachedNtdllThunk =
@@ -1699,50 +1705,28 @@ RefPtr<ProcessHandlePromise> WindowsProcessLauncher::DoLaunch() {
     Result<Ok, LaunchError> err = mResults.mSandboxBroker->LaunchApp(
         mCmdLine->program().c_str(), mCmdLine->command_line_string().c_str(),
         mLaunchOptions->env_map, mProcessType, mEnableSandboxLogging,
-        cachedNtdllThunk, &handle);
+        cachedNtdllThunk, &mResults.mHandle);
     if (err.isOk()) {
       EnvironmentLog("MOZ_PROCESS_LOG")
           .print("==> process %d launched child process %d (%S)\n",
-                 base::GetCurrentProcId(), base::GetProcId(handle),
+                 base::GetCurrentProcId(), base::GetProcId(mResults.mHandle),
                  mCmdLine->command_line_string().c_str());
-      return ProcessHandlePromise::CreateAndResolve(handle, __func__);
+      return ProcessLaunchPromise::CreateAndResolve(std::move(mResults),
+                                                    __func__);
     }
-    return ProcessHandlePromise::CreateAndReject(err.unwrapErr(), __func__);
+    return ProcessLaunchPromise::CreateAndReject(err.unwrapErr(), __func__);
   }
 #  endif  // defined(MOZ_SANDBOX)
 
   Result<Ok, LaunchError> launchErr =
-      base::LaunchApp(mCmdLine.ref(), *mLaunchOptions, &handle);
+      base::LaunchApp(mCmdLine.ref(), *mLaunchOptions, &mResults.mHandle);
   if (launchErr.isErr()) {
-    return ProcessHandlePromise::CreateAndReject(launchErr.unwrapErr(),
+    return ProcessLaunchPromise::CreateAndReject(launchErr.unwrapErr(),
                                                  __func__);
   }
-  return ProcessHandlePromise::CreateAndResolve(handle, __func__);
-}
-
-Result<Ok, LaunchError> WindowsProcessLauncher::DoFinishLaunch() {
-  Result<Ok, LaunchError> err = BaseProcessLauncher::DoFinishLaunch();
-  if (err.isErr()) {
-    return err;
-  }
-
-  return Ok();
-}
-#endif  // XP_WIN
-
-RefPtr<ProcessLaunchPromise> BaseProcessLauncher::FinishLaunch() {
-  Result<Ok, LaunchError> aError = DoFinishLaunch();
-  if (aError.isErr()) {
-    return ProcessLaunchPromise::CreateAndReject(aError.unwrapErr(), __func__);
-  }
-
-  MOZ_DIAGNOSTIC_ASSERT(mResults.mHandle);
-
-  glean::process::child_launch.AccumulateRawDuration(TimeStamp::Now() -
-                                                     mStartTimeStamp);
-
   return ProcessLaunchPromise::CreateAndResolve(std::move(mResults), __func__);
 }
+#endif  // XP_WIN
 
 bool GeckoChildProcessHost::OpenPrivilegedHandle(base::ProcessId aPid) {
   if (mChildProcessHandle) {
@@ -1855,6 +1839,9 @@ bool GeckoChildProcessHost::StartMacSandbox(int aArgc, char** aArgv,
     case GeckoProcessType_Utility:
       sandboxType = ipc::UtilityProcessHost::GetMacSandboxType();
       break;
+    case GeckoProcessType_GPU:
+      sandboxType = gfx::GPUProcessHost::GetMacSandboxType();
+      break;
     default:
       return true;
   }
