@@ -9,8 +9,6 @@
 #include "builtin/intl/Segmenter.h"
 
 #include "mozilla/Assertions.h"
-#include "mozilla/IntegerTypeTraits.h"
-#include "mozilla/Range.h"
 #include "mozilla/UniquePtr.h"
 
 #include "jspubtd.h"
