@@ -10,14 +10,11 @@
 #include "mozilla/intl/TimeZone.h"
 #include "mozilla/Likely.h"
 #include "mozilla/Maybe.h"
-#include "mozilla/Range.h"
-#include "mozilla/Result.h"
 #include "mozilla/Span.h"
 #include "mozilla/UniquePtr.h"
 
 #include <cmath>
 #include <cstdlib>
-#include <iterator>
 #include <string_view>
 #include <utility>
 
