@@ -20,15 +20,12 @@
 #include "certdb.h"
 #include "mozilla/AppShutdown.h"
 #include "mozilla/Assertions.h"
-#include "mozilla/Casting.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Logging.h"
-#include "mozilla/PodOperations.h"
 #include "mozilla/Services.h"
 #include "mozilla/StaticPrefs_security.h"
 #include "mozilla/SyncRunnable.h"
 #include "mozilla/TimeStamp.h"
-#include "mozilla/Unused.h"
 #include "mozilla/glean/SecurityCertverifierMetrics.h"
 #include "mozpkix/Result.h"
 #include "mozpkix/pkix.h"
@@ -40,6 +37,7 @@
 #include "nsNSSCallbacks.h"
 #include "nsNSSCertificate.h"
 #include "nsNSSCertificateDB.h"
+#include "nsNSSComponent.h"
 #include "nsNSSIOLayer.h"
 #include "nsNetCID.h"
 #include "nsPrintfCString.h"
@@ -63,7 +61,7 @@ namespace mozilla {
 namespace psm {
 
 NSSCertDBTrustDomain::NSSCertDBTrustDomain(
-    SECTrustType certDBTrustType, OCSPFetching ocspFetching,
+    SECTrustType certDBTrustType, RevocationCheckMode ocspFetching,
     OCSPCache& ocspCache, SignatureCache* signatureCache,
     TrustCache* trustCache, /*optional but shouldn't be*/ void* pinArg,
     TimeDuration ocspTimeoutSoft, TimeDuration ocspTimeoutHard,
@@ -600,20 +598,19 @@ Result NSSCertDBTrustDomain::DigestBuf(Input item, DigestAlgorithm digestAlg,
 
 TimeDuration NSSCertDBTrustDomain::GetOCSPTimeout() const {
   switch (mOCSPFetching) {
-    case NSSCertDBTrustDomain::FetchOCSPForDVSoftFail:
+    case NSSCertDBTrustDomain::RevocationCheckMayFetch:
       return mOCSPTimeoutSoft;
-    case NSSCertDBTrustDomain::FetchOCSPForEV:
-    case NSSCertDBTrustDomain::FetchOCSPForDVHardFail:
+    case NSSCertDBTrustDomain::RevocationCheckRequired:
       return mOCSPTimeoutHard;
-    // The rest of these are error cases. Assert in debug builds, but return
-    // the soft timeout value in release builds.
-    case NSSCertDBTrustDomain::NeverFetchOCSP:
-    case NSSCertDBTrustDomain::LocalOnlyOCSPForEV:
-      MOZ_ASSERT_UNREACHABLE("we should never see this OCSPFetching type here");
+    // Reaching this case is an error. Assert in debug builds, but return the
+    // soft timeout value in release builds.
+    case NSSCertDBTrustDomain::RevocationCheckLocalOnly:
+      MOZ_ASSERT_UNREACHABLE(
+          "we should never see this RevocationCheckMode type here");
       break;
   }
 
-  MOZ_ASSERT_UNREACHABLE("we're not handling every OCSPFetching type");
+  MOZ_ASSERT_UNREACHABLE("we're not handling every RevocationCheckMode type");
   return mOCSPTimeoutSoft;
 }
 
@@ -747,22 +744,6 @@ Result NSSCertDBTrustDomain::CheckRevocation(
     return Success;
   }
 
-  // Look for an OCSP Authority Information Access URL. Our behavior in
-  // ConfirmRevocations mode depends on whether a synchronous OCSP
-  // request is possible.
-  nsCString aiaLocation(VoidCString());
-  if (aiaExtension) {
-    UniquePLArenaPool arena(PORT_NewArena(DER_DEFAULT_CHUNKSIZE));
-    if (!arena) {
-      return Result::FATAL_ERROR_NO_MEMORY;
-    }
-    Result rv =
-        GetOCSPAuthorityInfoAccessLocation(arena, *aiaExtension, aiaLocation);
-    if (rv != Success) {
-      return rv;
-    }
-  }
-
   bool crliteCoversCertificate = false;
   Result crliteResult = Success;
   if (mCRLiteMode != CRLiteMode::Disabled) {
@@ -779,41 +760,28 @@ Result NSSCertDBTrustDomain::CheckRevocation(
     if (crliteCoversCertificate) {
       mozilla::glean::cert_verifier::cert_revocation_mechanisms.Get("CRLite"_ns)
           .Add(1);
-      // If we don't return here we will consult OCSP.
-      // In Enforce CRLite mode we can return "Revoked" or "Not Revoked"
-      // without consulting OCSP.
       if (mCRLiteMode == CRLiteMode::Enforce) {
         return crliteResult;
       }
-      // If we don't have a URL for an OCSP responder, then we can return any
-      // result ConfirmRevocations mode. Note that we might have a
-      // stapled or cached OCSP response which we ignore in this case.
-      if (mCRLiteMode == CRLiteMode::ConfirmRevocations &&
-          aiaLocation.IsVoid()) {
-        return crliteResult;
-      }
-      // In ConfirmRevocations mode we can return "Not Revoked"
-      // without consulting OCSP.
-      if (mCRLiteMode == CRLiteMode::ConfirmRevocations &&
-          crliteResult == Success) {
-        return Success;
-      }
     }
   }
 
-  bool ocspSoftFailure = false;
+  nsCString aiaLocation(VoidCString());
+  if (aiaExtension) {
+    UniquePLArenaPool arena(PORT_NewArena(DER_DEFAULT_CHUNKSIZE));
+    if (!arena) {
+      return Result::FATAL_ERROR_NO_MEMORY;
+    }
+    Result rv =
+        GetOCSPAuthorityInfoAccessLocation(arena, *aiaExtension, aiaLocation);
+    if (rv != Success) {
+      return rv;
+    }
+  }
+
   Result ocspResult = CheckRevocationByOCSP(
       certID, time, validityDuration, aiaLocation, crliteCoversCertificate,
-      crliteResult, stapledOCSPResponse, ocspSoftFailure);
-
-  // In ConfirmRevocations mode we treat any OCSP failure as confirmation
-  // of a CRLite revoked result.
-  if (crliteCoversCertificate &&
-      crliteResult == Result::ERROR_REVOKED_CERTIFICATE &&
-      mCRLiteMode == CRLiteMode::ConfirmRevocations &&
-      (ocspResult != Success || ocspSoftFailure)) {
-    return Result::ERROR_REVOKED_CERTIFICATE;
-  }
+      crliteResult, stapledOCSPResponse);
 
   MOZ_LOG(gCertVerifierLog, LogLevel::Debug,
           ("NSSCertDBTrustDomain: end of CheckRevocation"));
@@ -827,10 +795,40 @@ Result NSSCertDBTrustDomain::CheckRevocationByCRLite(
   MOZ_LOG(gCertVerifierLog, LogLevel::Debug,
           ("NSSCertDBTrustDomain::CheckRevocation: checking CRLite"));
 
-  // CRLite relies on timestamps from SCTs, so we need to check signatures on
-  // SCTs before calling into CRLite. The result only depends on the end entity
-  // certificate and the issuer SPKI, so it is path-independent and we only need
-  // to compute it once.
+  nsTArray<uint8_t> issuerSubjectPublicKeyInfoBytes;
+  issuerSubjectPublicKeyInfoBytes.AppendElements(
+      certID.issuerSubjectPublicKeyInfo.UnsafeGetData(),
+      certID.issuerSubjectPublicKeyInfo.GetLength());
+  nsTArray<uint8_t> serialNumberBytes;
+  serialNumberBytes.AppendElements(certID.serialNumber.UnsafeGetData(),
+                                   certID.serialNumber.GetLength());
+
+  nsTArray<RefPtr<nsICRLiteTimestamp>> timestamps;
+
+  // CRLite relies on timestamps from SCTs, so we should check signatures on
+  // SCTs before calling into CRLite. However, the risk of using unverified
+  // timestamps (particularly from embedded SCTs) is marginal, and if CT is
+  // disabled we will pass unverified timestamps from embedded SCTs to CRLite.
+  if (GetCertificateTransparencyMode() ==
+      CertVerifier::CertificateTransparencyMode::Disabled) {
+    size_t decodingErrors;
+    std::vector<SignedCertificateTimestamp> decodedSCTsFromExtension;
+    DecodeSCTs(GetSCTListFromCertificate(), decodedSCTsFromExtension,
+               decodingErrors);
+    (void)decodingErrors;
+    for (const auto& sct : decodedSCTsFromExtension) {
+      timestamps.AppendElement(new CRLiteTimestamp(sct));
+    }
+
+    return CheckCRLite(issuerSubjectPublicKeyInfoBytes, serialNumberBytes,
+                       timestamps, crliteCoversCertificate);
+  }
+
+  // When CT is enabled, we verify the signatures on all available SCTs and
+  // cache the verification result in the trust domain so that it can be used
+  // for CT policy enforcement. The verification result only depends on the end
+  // entity certificate and the issuer SPKI, so it is path-independent and we
+  // only need to compute it once.
   if (mCTVerifyResult.isNothing()) {
     MOZ_ASSERT(mBuiltChain.Length() > 0);
 
@@ -858,15 +856,6 @@ Result NSSCertDBTrustDomain::CheckRevocationByCRLite(
     mCTVerifyResult.emplace(std::move(ctVerifyResult));
   }
 
-  nsTArray<uint8_t> issuerSubjectPublicKeyInfoBytes;
-  issuerSubjectPublicKeyInfoBytes.AppendElements(
-      certID.issuerSubjectPublicKeyInfo.UnsafeGetData(),
-      certID.issuerSubjectPublicKeyInfo.GetLength());
-  nsTArray<uint8_t> serialNumberBytes;
-  serialNumberBytes.AppendElements(certID.serialNumber.UnsafeGetData(),
-                                   certID.serialNumber.GetLength());
-
-  nsTArray<RefPtr<nsICRLiteTimestamp>> timestamps;
   if (mCTVerifyResult.isSome()) {
     for (const auto& sct : mCTVerifyResult->verifiedScts) {
       timestamps.AppendElement(new CRLiteTimestamp(sct));
@@ -881,9 +870,7 @@ Result NSSCertDBTrustDomain::CheckRevocationByOCSP(
     const CertID& certID, Time time, Duration validityDuration,
     const nsCString& aiaLocation, const bool crliteCoversCertificate,
     const Result crliteResult,
-    /*optional*/ const Input* stapledOCSPResponse,
-    /*out*/ bool& softFailure) {
-  softFailure = false;
+    /*optional*/ const Input* stapledOCSPResponse) {
   const uint16_t maxOCSPLifetimeInDays = 10;
   // If we have a stapled OCSP response then the verification of that response
   // determines the result unless the OCSP response is expired. We make an
@@ -1002,7 +989,8 @@ Result NSSCertDBTrustDomain::CheckRevocationByOCSP(
         .Get("ShortValidity"_ns)
         .Add(1);
   }
-  if ((mOCSPFetching == NeverFetchOCSP) || (validityDuration < shortLifetime)) {
+  if ((mOCSPFetching == RevocationCheckLocalOnly) ||
+      (validityDuration < shortLifetime)) {
     // We're not going to be doing any fetching, so if there was a cached
     // "unknown" response, say so.
     if (cachedResponseResult == Result::ERROR_OCSP_UNKNOWN_CERT) {
@@ -1010,12 +998,11 @@ Result NSSCertDBTrustDomain::CheckRevocationByOCSP(
     }
     // If we're doing hard-fail, we want to know if we have a cached response
     // that has expired.
-    if (mOCSPFetching == FetchOCSPForDVHardFail &&
+    if (mOCSPFetching == RevocationCheckRequired &&
         cachedResponseResult == Result::ERROR_OCSP_OLD_RESPONSE) {
       return Result::ERROR_OCSP_OLD_RESPONSE;
     }
 
-    softFailure = true;
     return Success;
   }
 
@@ -1024,27 +1011,18 @@ Result NSSCertDBTrustDomain::CheckRevocationByOCSP(
   //  1) the user has not yet downloaded CRLite filters, or
   //  2) the user's CRLite filters are out-of-date, or
   //  3) the certificate has been in CT for < 1 MMD interval.
-  // If we're configured to enforce CRLite (i.e. CRLite is enabled and it is not
-  // in "confirm revocations" mode) and we're configured to tolerate OCSP soft
-  // failures, then it's reasonable to skip the synchronous OCSP request here.
-  // In effect, we're choosing to preserve the privacy of the user at the risk
-  // of potentially allowing them to navigate to a site that is serving a
+  // If we're configured to enforce CRLite and we're configured to tolerate OCSP
+  // soft failures, then it's reasonable to skip the synchronous OCSP request
+  // here. In effect, we're choosing to preserve the privacy of the user at the
+  // risk of potentially allowing them to navigate to a site that is serving a
   // revoked certificate.
-  if (mCRLiteMode == CRLiteMode::Enforce &&
-      mOCSPFetching == FetchOCSPForDVSoftFail && mIsBuiltChainRootBuiltInRoot) {
+  if (mOCSPFetching == RevocationCheckMayFetch &&
+      mCRLiteMode == CRLiteMode::Enforce && mIsBuiltChainRootBuiltInRoot) {
     return Success;
   }
 
-  if (mOCSPFetching == LocalOnlyOCSPForEV) {
-    if (cachedResponseResult != Success) {
-      return cachedResponseResult;
-    }
-    return Result::ERROR_OCSP_UNKNOWN_CERT;
-  }
-
   if (aiaLocation.IsVoid()) {
-    if (mOCSPFetching == FetchOCSPForEV ||
-        cachedResponseResult == Result::ERROR_OCSP_UNKNOWN_CERT) {
+    if (cachedResponseResult == Result::ERROR_OCSP_UNKNOWN_CERT) {
       return Result::ERROR_OCSP_UNKNOWN_CERT;
     }
     if (cachedResponseResult == Result::ERROR_OCSP_OLD_RESPONSE) {
@@ -1057,9 +1035,7 @@ Result NSSCertDBTrustDomain::CheckRevocationByOCSP(
     // Nothing to do if we don't have an OCSP responder URI for the cert; just
     // assume it is good. Note that this is the confusing, but intended,
     // interpretation of "strict" revocation checking in the face of a
-    // certificate that lacks an OCSP responder URI. There's no need to set
-    // softFailure here---we check for the presence of an AIA before attempting
-    // OCSP when CRLite is configured in confirm revocations mode.
+    // certificate that lacks an OCSP responder URI.
     return Success;
   }
 
@@ -1071,19 +1047,18 @@ Result NSSCertDBTrustDomain::CheckRevocationByOCSP(
     // responses from a failing server.
     return SynchronousCheckRevocationWithServer(
         certID, aiaLocation, time, maxOCSPLifetimeInDays, cachedResponseResult,
-        stapledOCSPResponseResult, crliteCoversCertificate, crliteResult,
-        softFailure);
+        stapledOCSPResponseResult, crliteCoversCertificate, crliteResult);
   }
 
   return HandleOCSPFailure(cachedResponseResult, stapledOCSPResponseResult,
-                           cachedResponseResult, softFailure);
+                           cachedResponseResult);
 }
 
 Result NSSCertDBTrustDomain::SynchronousCheckRevocationWithServer(
     const CertID& certID, const nsCString& aiaLocation, Time time,
     uint16_t maxOCSPLifetimeInDays, const Result cachedResponseResult,
     const Result stapledOCSPResponseResult, const bool crliteCoversCertificate,
-    const Result crliteResult, /*out*/ bool& softFailure) {
+    const Result crliteResult) {
   if (AppShutdown::IsInOrBeyond(ShutdownPhase::AppShutdownConfirmed)) {
     return Result::FATAL_ERROR_LIBRARY_FAILURE;
   }
@@ -1129,7 +1104,7 @@ Result NSSCertDBTrustDomain::SynchronousCheckRevocationWithServer(
     }
 
     return HandleOCSPFailure(cachedResponseResult, stapledOCSPResponseResult,
-                             rv, softFailure);
+                             rv);
   }
 
   // If the response from the network has expired but indicates a revoked
@@ -1167,7 +1142,7 @@ Result NSSCertDBTrustDomain::SynchronousCheckRevocationWithServer(
     }
   }
 
-  if (rv == Success || mOCSPFetching != FetchOCSPForDVSoftFail) {
+  if (rv == Success || mOCSPFetching == RevocationCheckRequired) {
     MOZ_LOG(gCertVerifierLog, LogLevel::Debug,
             ("NSSCertDBTrustDomain: returning after "
              "VerifyEncodedOCSPResponse"));
@@ -1186,14 +1161,13 @@ Result NSSCertDBTrustDomain::SynchronousCheckRevocationWithServer(
     return stapledOCSPResponseResult;
   }
 
-  softFailure = true;
   return Success;  // Soft fail -> success :(
 }
 
 Result NSSCertDBTrustDomain::HandleOCSPFailure(
     const Result cachedResponseResult, const Result stapledOCSPResponseResult,
-    const Result error, /*out*/ bool& softFailure) {
-  if (mOCSPFetching != FetchOCSPForDVSoftFail) {
+    const Result error) {
+  if (mOCSPFetching == RevocationCheckRequired) {
     MOZ_LOG(gCertVerifierLog, LogLevel::Debug,
             ("NSSCertDBTrustDomain: returning SECFailure after OCSP request "
              "failure"));
@@ -1218,7 +1192,6 @@ Result NSSCertDBTrustDomain::HandleOCSPFailure(
           ("NSSCertDBTrustDomain: returning SECSuccess after OCSP request "
            "failure"));
 
-  softFailure = true;
   return Success;  // Soft fail -> success :(
 }
 
@@ -1539,7 +1512,7 @@ static Input SECItemToInput(const UniqueSECItem& item) {
     // As used here, |item| originally comes from an Input,
     // so there should be no issues converting it back.
     MOZ_ASSERT(rv == Success);
-    Unused << rv;  // suppresses warnings in release builds
+    (void)rv;  // suppresses warnings in release builds
   }
   return result;
 }
@@ -1617,7 +1590,7 @@ SECStatus InitializeNSS(const nsACString& dir, NSSDBConfig nssDbConfig,
     if (PK11_NeedUserInit(slot.get())) {
       srv = PK11_InitPin(slot.get(), nullptr, nullptr);
       MOZ_ASSERT(srv == SECSuccess);
-      Unused << srv;
+      (void)srv;
     }
   }
 
@@ -1649,7 +1622,7 @@ bool LoadUserModuleAt(const char* moduleName, const char* libraryName,
   // it. Note that it isn't possible to delete the internal module, so checking
   // the return value would be detrimental in that case.
   int unusedModType;
-  Unused << SECMOD_DeleteModule(moduleName, &unusedModType);
+  (void)SECMOD_DeleteModule(moduleName, &unusedModType);
 
   nsAutoCString fullLibraryPath;
   if (!dir.IsEmpty()) {
@@ -1693,7 +1666,7 @@ bool LoadUserModuleFromXul(const char* moduleName,
   // it. Note that it isn't possible to delete the internal module, so checking
   // the return value would be detrimental in that case.
   int unusedModType;
-  Unused << SECMOD_DeleteModule(moduleName, &unusedModType);
+  (void)SECMOD_DeleteModule(moduleName, &unusedModType);
 
   UniqueSECMODModule userModule(
       SECMOD_LoadUserModuleWithFunction(moduleName, fentry));
@@ -1760,7 +1733,7 @@ bool LoadOSClientCertsModule() {
 
 bool LoadLoadableRoots(const nsCString& dir) {
   int unusedModType;
-  Unused << SECMOD_DeleteModule("Root Certs", &unusedModType);
+  (void)SECMOD_DeleteModule("Root Certs", &unusedModType);
   return LoadUserModuleAt(kRootModuleName.get(), "nssckbi", dir, nullptr);
 }
 
@@ -1778,7 +1751,7 @@ bool LoadLoadableRootsFromXul() {
   // can cause us to fail to load our roots module. In these cases, deleting the
   // "Root Certs" module allows us to load the correct one. See bug 1406396.
   int unusedModType;
-  Unused << SECMOD_DeleteModule("Root Certs", &unusedModType);
+  (void)SECMOD_DeleteModule("Root Certs", &unusedModType);
 
   if (!LoadUserModuleFromXul(kRootModuleName.get(),
                              TRUST_ANCHORS_GetFunctionList)) {
