@@ -60,6 +60,12 @@ static const char kBackgroundPageHTMLEnd[] =
   "extensions.webextensions.base-content-security-policy.v3"
 #define DEFAULT_BASE_CSP_V3 "script-src 'self' 'wasm-unsafe-eval';"
 
+#define BASE_CSP_PREF_V3_WITH_LOCALHOST \
+  "extensions.webextensions.base-content-security-policy.v3-with-localhost"
+#define DEFAULT_BASE_CSP_V3_WITH_LOCALHOST                   \
+  "script-src 'self' 'wasm-unsafe-eval' http://localhost:* " \
+  "http://127.0.0.1:*;"
+
 static inline ExtensionPolicyService& EPS() {
   return ExtensionPolicyService::GetSingleton();
 }
@@ -191,6 +197,7 @@ WebExtensionPolicyCore::WebExtensionPolicyCore(GlobalObject& aGlobal,
       mTemporarilyInstalled(aInit.mTemporarilyInstalled),
       mBackgroundWorkerScript(aInit.mBackgroundWorkerScript),
       mIgnoreQuarantine(aInit.mIsPrivileged || aInit.mIgnoreQuarantine),
+      mHasRecommendedState(aInit.mHasRecommendedState),
       mPermissions(new AtomSet(aInit.mPermissions)) {
   // In practice this is not necessary, but in tests where the uuid
   // passed in is not lowercased various tests can fail.
@@ -202,6 +209,13 @@ WebExtensionPolicyCore::WebExtensionPolicyCore(GlobalObject& aGlobal,
     if (NS_FAILED(rv)) {
       mBaseCSP = NS_LITERAL_STRING_FROM_CSTRING(DEFAULT_BASE_CSP_V2);
     }
+  } else if (mTemporarilyInstalled) {
+    nsresult rv =
+        Preferences::GetString(BASE_CSP_PREF_V3_WITH_LOCALHOST, mBaseCSP);
+    if (NS_FAILED(rv)) {
+      mBaseCSP =
+          NS_LITERAL_STRING_FROM_CSTRING(DEFAULT_BASE_CSP_V3_WITH_LOCALHOST);
+    }
   } else {
     nsresult rv = Preferences::GetString(BASE_CSP_PREF_V3, mBaseCSP);
     if (NS_FAILED(rv)) {
@@ -402,7 +416,7 @@ bool WebExtensionPolicy::Enable() {
     mBrowsingContextGroup = group->MakeKeepAlivePtr();
   }
 
-  Unused << Proto()->SetSubstitution(MozExtensionHostname(), BaseURI());
+  (void)Proto()->SetSubstitution(MozExtensionHostname(), BaseURI());
 
   mActive = true;
   return true;
@@ -422,7 +436,7 @@ bool WebExtensionPolicy::Disable() {
     mBrowsingContextGroup = nullptr;
   }
 
-  Unused << Proto()->SetSubstitution(MozExtensionHostname(), nullptr);
+  (void)Proto()->SetSubstitution(MozExtensionHostname(), nullptr);
 
   mActive = false;
   return true;
@@ -455,6 +469,10 @@ void WebExtensionPolicy::SetIgnoreQuarantine(bool aIgnore) {
   mCore->SetIgnoreQuarantine(aIgnore);
 }
 
+void WebExtensionPolicy::SetHasRecommendedState(bool aHasRecommendedState) {
+  mCore->SetHasRecommendedState(aHasRecommendedState);
+}
+
 void WebExtensionPolicy::RegisterContentScript(
     WebExtensionContentScript& script, ErrorResult& aRv) {
   // Raise an "invalid argument" error if the script is not related to
@@ -554,7 +572,7 @@ bool WebExtensionPolicy::IsQuarantinedDoc(const DocInfo& aDoc) {
 /* static */
 bool WebExtensionPolicy::IsQuarantinedURI(const URLInfo& aURI) {
   // Ensure EPS is initialized before asking it about quarantined domains.
-  Unused << EPS();
+  (void)EPS();
 
   RefPtr<AtomSet> quarantinedDomains =
       ExtensionPolicyService::QuarantinedDomains();
@@ -1006,9 +1024,7 @@ void DocumentObserver::Observe(
   }
 }
 
-void DocumentObserver::Disconnect() {
-  Unused << EPS().UnregisterObserver(*this);
-}
+void DocumentObserver::Disconnect() { (void)EPS().UnregisterObserver(*this); }
 
 void DocumentObserver::NotifyMatch(MozDocumentMatcher& aMatcher,
                                    nsPIDOMWindowOuter* aWindow) {
